package control;

/* Charge Control Version 1.1 (c) 2008 by Malte Marwedel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import gnu.io.CommPort;

import java.io.IOException;
import java.util.Vector;

/**
 * This is a thread for communicating with the serial cable.
 * It provides a status value for the charger and all incoming bytes are placed into a line based FIFO
 * Moreover commands for reading and writing the configuration are provided
 * 
 * This class implements the singleton pattern
 * 
 * @author Malte Marwedel
 *
 */

public class SerialThread implements Runnable {
	
	private static SerialThread st;
	
	private final int SAMPLEDURATION = 10000;
	private final int POLL_FREQ_MS = 1;
	

	private volatile Vector<byte[]> fifo;
	

	private volatile boolean shouldRun = true;
	private volatile boolean iamRunning = false;
	
	private Thread sthr;
	
	private volatile CommPort commport;
	
	//some variables to tell the thread what he should do
	private volatile boolean startSample;
	private volatile int currentSample;
	private final int maxSample = 131072;
	
	
	public static SerialThread getInstance() {
		if (st == null) {
			st = new SerialThread();
		}
		return st;
	}
	
	private SerialThread() {
		fifo = new Vector<byte[]>();
	}
	
	public void run() {
		iamRunning = true;
		String accumu = new String();
		while(shouldRun) {
			int avar;
			try {
				avar = commport.getInputStream().available();
				//System.out.println("Available on input "+avar);
				if (avar > 0) {
					byte arr[] = new byte[avar];
					commport.getInputStream().read(arr, 0, avar);
					putToFifo(arr);
					System.out.print("\rSerialThread: Got data byte "+currentSample);
					currentSample++;
					if ((currentSample < maxSample) && (!startSample)) {
						byte a[] = new byte[1]; 
						if (currentSample%2 == 0) { //first sample done, second request
							a[0] = 'a';
						} else
							a[0] = 's';
						commport.getOutputStream().write(a);
					}
				}
				if (startSample) {
					startSample = false;
					currentSample = 0;
					byte a[] = new byte[1];
					a[0] = 'd';
					System.out.println("SerialThread.run: Starting sampling");
					commport.getOutputStream().write(a); //start recording
					try {
	          Thread.sleep(SAMPLEDURATION);
          } catch (InterruptedException e) {
	          //who cares
          	e.printStackTrace();
          }
					a[0] = 'a';
					commport.getOutputStream().write(a); //request first sampled value
				}
			} catch (IOException e) {
				avar = 0;
				System.out.println("Serial.Thread.run: Error: "+e.getMessage());
			}
			try {
	      Thread.sleep(POLL_FREQ_MS);
      } catch (InterruptedException e) {
	      //who cares
      }
		}
		iamRunning = false;
	}
	
	public void start(CommPort cp) {
		commport = cp;
		shouldRun = true;
		sthr = new Thread(this);
		sthr.start();
		while(!iamRunning) {			
			try {
	      Thread.sleep(1);
      } catch (InterruptedException e) {
	      //who cares
      }
		}
	}
	
	public void stop() {
		shouldRun = false;
		while(iamRunning) {
			try {
	      Thread.sleep(1);
      } catch (InterruptedException e) {
	      //who cares
      }
		}
	}
	
	public String getChargerstateDescription() {
		return "unknown"; //not used for the DSO
	}

	public int getChargerstate() {
		return 0;
	}
	
	public synchronized int getFifoSize() {
		return fifo.size();
	}
	
	private synchronized void putToFifo(byte[] arr) {
		fifo.add(arr);
	}
	
	public synchronized byte[] getFromFifo() {
		if (fifo.size() > 0) {
			byte[] arr = fifo.get(0);
			fifo.remove(0);
			return arr;
		}
		System.out.println("SerialThread.getFromFifo: Warning: Tried to read from FIFO, but FIFO is empty");
		return null;
	}

	public void sampleData() {
		startSample = true;
	}
}
